@@ -30,7 +30,7 @@ tuntap_pre_start()
 		local timeout=10
 		while [ ! -e /dev/net/tun -a ${timeout} -gt 0 ]; do
 			sleep 1
-			timeout=$((${timeout} - 1))
+			: $(( timeout -= 1 ))
 		done
 		if [ ! -e /dev/net/tun ]; then
 			eerror "TUN/TAP support present but /dev/net/tun is not"
@@ -54,15 +54,18 @@ tuntap_pre_start()
 		do_tunctl=true
 	elif type openvpn >/dev/null 2>&1; then
 		do_openvpn=true
-	else
+	elif type tunctl >/dev/null 2>&1; then
 		do_tunctl=true
 	fi
 
 	if ${do_openvpn}; then
 		openvpn --mktun --dev-type "${tuntap}" --dev "${IFACE}" \
 			${o_opts} >/dev/null
-	else
+	elif ${do_tunctl}; then
 		tunctl ${t_opts} -t "${IFACE}" >/dev/null
+	else
+		eerror "Neither openvpn nor tunctl has been found, please install"
+		eerror "either \"openvpn\" or \"usermode-utilities\"."
 	fi
 	eend $? && _up && service_set_value tuntap "${tuntap}"
 }
